<?php
$PAGE_TYPES_TEXTBASE = array();
define('PAGE_WRITE_ERROR_NONE',        0);
define('PAGE_WRITE_ERROR_FAIL',        1);
define('PAGE_WRITE_ERROR_BACKUP',      2);
define('PAGE_WRITE_ERROR_IO',          3);
define('PAGE_WRITE_ERROR_CONFLICT',    4);

function page_get_pageid($pagename) {
    return ascii_encode($pagename);
}

function page_get_pagename($pageid) {
    return ascii_decode($pageid);
}

function page_get_simplename($pagename) {
    $names = explode('/', $pagename);
    return end($names);
}

function page_is_exists($pagename) {
    return storage_page_is_exists($pagename);
}

function page_is_deleted($pagename) {
    return storage_page_is_deleted($pagename);
}

function page_is_system_page($pagename) {
    return $pagename === MANUAL_PAGENAME ||
	strpos($pagename, MANUAL_PAGENAME . '/') === 0;
}

function page_create($pagename) {
    $page = array();
    $page['name'] = $pagename;
    $page['id'] = page_get_pageid($pagename);
    $page['meta'] = array();
    $page['is_exists'] = false;
    return $page;
}

function page_setup(&$page) {
    if(empty($page['meta']['type']))
	$page['meta']['type'] = DEFAULT_PAGE_TYPE;
}

function &page_read($pagename) {
    auth_check($pagename, 'read');
    $page = page_create($pagename);
    storage_page_read($page);
    page_setup($page);
    return $page;
}

function page_read_backup($pagename, $backuptime) {
    auth_check($pagename, 'read');
    $page = page_create($pagename);
    storage_page_read_backup($page, $backuptime);
    page_setup($page);
    $page['backuptime'] = $backuptime;
    return $page;
}

function page_open_contents(&$page) {
    return storage_page_open_contents($page);
}

function page_get_contents(&$page) {
    $fp = page_open_contents($page);
    if($fp === false)
	return false;
    $contents = stream_get_contents($fp);
    fclose($fp);
    return $contents;
}

function page_output_contents(&$page) {
    $fp = page_open_contents($page);
    if($fp === false)
	return false;
    while(!feof($fp))
	echo(fread($fp, CONTENTS_READ_BUFFER_SIZE));
    fclose($fp);
}


function page_write(&$page, &$contents, &$ticket = '', &$error = PAGE_WRITE_ERROR_NONE) {
    if(!lock_has_permission($page) && !empty($page['meta']['lock']) && empty($page['lock_now'])) {
	message_error(l('This page was locked.'));
	return false;
    }

    if(!empty($page['insert']))
	auth_check($page['name'], 'insert');
    else
	auth_check($page['name'], 'write');
    $error = PAGE_WRITE_ERROR_NONE;

    if(!empty($page['is_exists']) && !empty($page['meta']['ticket'])) {
	if(empty($ticket) || $page['meta']['ticket'] != $ticket) {
	    message_error(l('Confliction detected.'));
	    $error = PAGE_WRITE_ERROR_CONFLICT;
	    return false;
	}
    }

    cache_delete($page['name']);
    cache_delete($page['name'], 'page_find_', true);

    $page['last_modify_user'] = default_value($page['meta']['last_modify_user'], '');
    if(($user = auth_get_user()) !== false) {
	$page['meta']['last_modify_user'] = $user['name'];
	if(empty($page['is_exists'])) {
	    $page['meta']['create_user'] = $user['name'];
	}
    } else {
	unset($page['meta']['last_modify_user']);
    }

    $page['meta']['ticket'] = md5($page['name'] . time());

    if(!isset($page['meta']['btime'])) {
	$page['meta']['btime'] = time();
    }

    $old_ngrams = search_page_ngram($page);
    if(isset($page['write_tag'])) {
	if($page['write_tag'] == '')
	    unset($page['meta']['tag']);
	else
	    $page['meta']['tag'] = $page['write_tag'];
    }
    $result = storage_page_write($page, $contents, $error);
    if($result) {
	$npage = page_read($page['name']);
	$new_ngrams = search_page_ngram($npage);
	search_index_update($page['name'], $old_ngrams, $new_ngrams);
    }
    return $result;
}

function page_delete(&$page) {
    if(!empty($page['meta']['lock'])) {
	message_error(l('This page was locked.'));
	return false;
    }
    auth_check($page['name'], 'write');
    cache_delete($page['name']);
    cache_delete($page['name'], 'page_find_', true);
    if(page_is_exists($page['name'])) {
	$old_ngrams = search_page_ngram($page);
	$result = storage_page_delete($page);
	if($result) {
	    search_index_remove($page['name'], $old_ngrams);
	}
	return $result;
    }
    return false;
}

function page_truncate(&$page) {
    auth_check($page['name'], 'admin');
    page_delete($page);
    return storage_page_truncate($page);
}

function page_lock(&$page, $username = '') {
    auth_check($page['name'], 'lock');

    if(isset($page['meta']['lock_user']) && $page['meta']['lock_user'] != $username) {
	message_error(l('This page was locked.'));
	return false;
    }

    $page['meta']['lock'] = true;
    $page['lock_now'] = true;

    if($username !== '')
	$page['meta']['lock_user'] = $username;
    $fp = page_open_contents($page);
    return page_write($page, $fp, $page['meta']['ticket']);
}

function page_unlock(&$page, $username = false) {
    if(empty($page['meta']['lock']))
	return true;
    auth_check($page['name'], 'lock');

    if(isset($page['meta']['lock_user']) &&
       (!auth_has_permission($page['name'], 'admin') && $page['meta']['lock_user'] != $username)) {
	message_error(lf('Only $user$ can unlock this page.', array('user' => $page['meta']['lock_user'])));
	return false;
    }
    unset($page['meta']['lock']);
    unset($page['meta']['lock_user']);
    $fp = page_open_contents($page);
    return page_write($page, $fp, $page['meta']['ticket']);
}

function page_rename($oldpagename, $newpagename, $is_rename_subpage = true) {
    auth_check($oldpagename, 'write');
    auth_check($newpagename, 'write');
    return storage_page_rename($oldpagename, page_path_to_pagename($newpagename), $is_rename_subpage);
}

function page_get_type(&$page) {
    if(isset($page['meta']['type']))
	return $page['meta']['type'];
    else
	return '';
}

function page_get_mtime($pagename) {
    return storage_page_get_mtime($pagename);
}

function page_type_is_text_base($page_type) {
    global $PAGE_TYPES_TEXTBASE;
    return in_array($page_type, $PAGE_TYPES_TEXTBASE);
}

function page_find($base_pagename = '', $condition = array()) {
    return storage_page_find($base_pagename, $condition);
}

function page_find_deleted($base_pagename = '', $condition = array()) {
    return storage_page_find_deleted($base_pagename, $condition);
}

function page_get_backup_times(&$page) {
    return storage_page_get_backup_times($page);
}

function page_get_backup_pages(&$page) {
    $backup_times = page_get_backup_times($page);
    $backup_pages = array();
    foreach($backup_times as $backup_time) {
	$backup_pages[$backup_time] = page_read_backup($page['name'], $backup_time);
    }
    return $backup_pages;
}

function page($pagename, &$page = null, &$args = null, $is_main = false) {
    global $page_stop_processing;
    if($page_stop_processing)
	return '';
    if(empty($page))
	$page = page_read($pagename);
    $page['args'] = &$args;
    $page['is_main'] = $is_main;

    $dom = dom_create_document();
    page_handle($page, $dom);
    return dom_save_html($dom);
}

function page_args(&$page) {
    if(isset($page['is_main']) && $page['is_main'] === false) { // side page
	return args_get();
    } else {
	if(isset($page['args']))
	    return $page['args'];
	else
	    return array();
    }
}

function page_stop_processing() {
    global $page_stop_processing;
    $page_stop_processing = true;
}

function page_handle(&$page, $dom, $action = '') {
    $page_dom = page_append_page_dom($page, $dom);
    handle($page, $page_dom, $action);
    if(!empty($page['meta']['ticket']))
	dom_set_attribute($page_dom, 'data-ticket', $page['meta']['ticket']);
    dom_remove_empty($page_dom);
}

function page_path_to_pagename($path, $basename = '', $enable_default_page = true) {
    $path = preg_replace('/[\x09\x0a]/', '', $path);
    $path = trim($path);
    $path = trim($path, '/');
    if(substr($path, 0, 1) == '.')
	$path = $basename . '/' . $path;
    $names = explode('/', $path);
    $result_names = array();
    foreach($names as $name) {
	if($name == '.' || $name == '') {
	} else if($name == '..') {
	    if(!empty($result_names))
		array_pop($result_names);
	} else {
	    $result_names[] = $name;
	}
    }
    if(empty($result_names) && $enable_default_page)
	return DEFAULT_PAGENAME;
    
    return implode('/', $result_names);
}

function page_get_relations($from_pagename, $to_pagename) {
    $relations = array();
    if($from_pagename == $to_pagename)
	$relations[] = 'self';
    else if(strpos($to_pagename, $from_pagename . '/') === 0) {
	if(dirname($to_pagename) == $from_pagename)
	    $relations[] = 'child';
	$relations[] = 'offspring';
    } else if(strpos($from_pagename, $to_pagename . '/') === 0) {
	if(dirname($from_pagename) == $to_pagename)
	    $relations[] = 'parent';
	$relations[] = 'ancestor';
    }
    return $relations;
}

function page_get_section_id($pagename, $sectionname) {
    return substr(md5($pagename . '#' . $sectionname), 0, 8);
}

function page_append_page_dom(&$page, $dom) {
    $page_element = dom_append_child($dom, 'section', 'page section',
				     array('data-pagename' => $page['name']));
    return $page_element;
}

class PageContentStream {
    var $pagename;
    var $fp;

    function stream_open($path, $mode, $options, &$opened_path) {
        $url = parse_url($path);
        $this->pagename = urldecode($url["host"]);
	$page = page_read($this->pagename);
        $this->fp = page_open_contents($page);
        return $this->fp !== false;
    }

    function stream_read($count) {
	return fread($this->fp, $count);
    }

    function stream_stat() {
        return fstat($this->fp);
    }

    function stream_tell() {
        return ftell($this->fp);
    }

    function stream_eof() {
        return feof($this->fp);
    }
}
stream_wrapper_register('twpage', 'PageContentStream');


?>
