<?php
define('START_MICROTIME', microtime(true));

function execution_time() {
    return microtime(true) - START_MICROTIME;
}

function args_get() {
    $args = $_GET + $_POST;
    foreach($args as $key => &$val) {
	if(is_string($val))
	    $val = normalize_string($val);
    }
    if(get_magic_quotes_gpc()) {
	foreach($args as $key => &$val) {
	    $val = stripslashes($val);
	}
    }
    return $args;
}

function normalize_string($binary) {
    $binary = preg_replace('/[\x00-\x08\x0b-\x1f\x7f]/', '', $binary);
    if(!preg_match('//u', $binary))
	return '';
    return $binary;
}

function hash_to_pairs($hash) {
    $pairs = array();
    foreach($hash as $k => $v) {
	$pairs[] = array($k, $v);
    }
    return $pairs;
}

function glob_array($pattern) {
    $result = array();
    $files = glob($pattern);
    if($files !== false) {
	foreach($files as $file) {
	    if(is_file($file)) {
		$result[] = $file;
	    }
	}
    }
    return $result;
}

function nextstop(&$arr) {
    $val = next($arr);
    if($val === false)
	end($arr);
    return $val;
}

function scalar_walk(&$v, $func, &$opt = null) {
    if(is_array($v)) {
	foreach($v as &$e)
	    scalar_walk($e, $func, $opt);
    } else {
	if(!is_null($opt))
	    $v = $func($v, $opt);
	else
	    $v = $func($v);
    }
}

function order_compare(&$a, &$b) {
    if($a['order'] < $b['order'])
	return -1;
    else if($a['order'] > $b['order'])
	return 1;
    return 0;
}

function ascii_encode($data) {
    $ar = unpack('H*', $data);
    return reset($ar);
    //return array_shift(unpack('H*', $data));
}

function ascii_decode($data) {
    return pack('H*', $data);
}

function decode_prefixed_number($val) {
    $val = trim($val, ' i');
    $l = strlen($val);
    if($l < 1)
	return $val;
    $last = strtolower($val[strlen($val)-1]);

    switch($last) {
    case 't':
	$val *= 1024;
    case 'g':
	$val *= 1024;
    case 'm':
	$val *= 1024;
    case 'k':
	$val *= 1024;
    }
    return $val;
}

function fraction_to_um($frac) {
    $parts = explode('/', $frac);
    if(count($parts) <= 0)
	return 0;
    if(count($parts) == 1)
	return $parts[0];
    return floatval($parts[0]) / floatval($parts[1]);
}

function encode_prefixed_number($val) {
    $prefix = array('', 'Ki', 'Mi', 'Gi', 'Ti');
    $p = 0;
    while($val >= 1024 && isset($prefix[$p])) {
	$val = floor($val/1024);
	$p++;
    }
    return $val . $prefix[$p];
}

function max_upload_file_size() {
    $max_upload = decode_prefixed_number(ini_get('upload_max_filesize'));
    $max_post = decode_prefixed_number(ini_get('post_max_size'));
    $memory_limit = decode_prefixed_number(ini_get('memory_limit'));
    return min($max_upload, $max_post, $memory_limit);
}

function default_value(&$var, $default = '') {
    if(isset($var))
	return $var;
    else
	return $default;
}

function pprint($msg) {
    print('<pre>' . htmlspecialchars($msg) . '</pre>');
}
function pprint_r(&$var) {
    pprint(print_r($var, true));
}

function email_is_valid($email) {
    return preg_match('/^[-+.\\w]+@[-a-z0-9]+(\\.[-a-z0-9]+)*\\.[a-z]{2,6}$/i', $email);
}

function color_is_valid($color) {
    $color = color_keyword_to_rgb($color);
    return preg_match('/^(#[0-9a-f]{3}|#[0-9a-f]{6})$/', $color);
}

function color_keyword_to_rgb($keyword) {
    global $COLOR_KEYWORDS;
    if(!isset($COLOR_KEYWORDS)) {
	require_once(APP_DIR_PATH . '/color_keywords.inc');
    }
    $keyword = strtolower($keyword);
    if(isset($COLOR_KEYWORDS[$keyword]))
	return $COLOR_KEYWORDS[$keyword];
    else
	return $keyword;
}

function color_html_to_rgb($color) {
    $color = color_keyword_to_rgb($color);
    if(strlen($color) == 4)
	$color = '#' . $color[1] . $color[1] . $color[2] . $color[2] . $color[3] . $color[3];
    if(preg_match('/^#([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/', $color, $maches)) {
	return array(
	    'r' => hexdec($maches[1]),
	    'g' => hexdec($maches[2]),
	    'b' => hexdec($maches[3]));
    } else {
	return false;
    }
}

function color_html_to_hsv($color) {
    $rgb = color_html_to_rgb($color);
    if($rgb === false)
	return false;
    return color_rgb_to_hsv($rgb);
}

function color_rgb_to_html($rgb) {
    return sprintf('#%02x%02x%02x', $rgb['r'], $rgb['g'], $rgb['b']);
}

function color_rgb_to_hsv($rgb) {
    $r = $rgb['r'] / 255;
    $g = $rgb['g'] / 255;
    $b = $rgb['b'] / 255;
    $max = max($r, $g, $b);
    $min = min($r, $g, $b);
    $v = $max;

    $diff = $max - $min;
    if($diff == 0){
	$h = 0;
    } else if($r === $max){
	$h = 60 * ($g - $b) / $diff + 0;
    } else if($g === $max){
	$h = 60 * ($b - $r) / $diff + 120;
    } else {
	$h = 60 * ($r - $g) / $diff + 240;
    }
    if($h < 0)
	$h = $h + 360;
 
    $s = ($v != 0) ? $diff / $max : 0;
   
    return array("h" => $h, "s" => $s, "v" => $v);
}

function color_hsv_to_rgb($hsv){
    $h = $hsv['h'];
    $s = $hsv['s'];
    $v = $hsv['v'];
    if($s == 0) {
	$r = $v * 255;
	$g = $v * 255;
	$b = $v * 255;
    } else {
	$h = ($h >= 0) ? $h % 360 / 360 : $h % 360 / 360 + 1;
	$var_h = $h * 6;
	$i = (int)$var_h;
	$f = $var_h - $i;
     
	$p = $v * ( 1 - $s );
	$q = $v * ( 1 - $s * $f );
	$t = $v * ( 1 - $s * ( 1 - $f ) );

	$a_r = array($v,$q,$p,$p,$t,$v);
	$a_g = array($t,$v,$v,$q,$p,$p);
	$a_b = array($p,$p,$t,$v,$v,$q);
	$r = round($a_r[$i] * 255);
	$g = round($a_g[$i] * 255);
	$b = round($a_b[$i] * 255);
    }
    return array('r' => $r, 'g' => $g, 'b' => $b);
}

function preg_match_table(&$patterns, $pattern_prefix, $pattern_postfix, $subject, &$maches = null) {
    foreach($patterns as $pattern => &$data) {
	if(preg_match($pattern_prefix . $pattern . $pattern_postfix, $subject, $maches)) {
	    return $data;
	}
    }
    return false;
}

function url_path_encode($pagename) {
    $names = explode('/', $pagename);
    $names = array_map('urlencode', $names);
    return implode('/', $names);
}

function url_make_query($h) {
    $records = array();
    foreach($h as $name => $value) {
	$records[] = urlencode($name) . '='. urlencode($value);
    }
    return implode('&', $records);
}

function url_parse_query($querystring) {
    $args = array();
    if($querystring == '')
	return $args;
    $fields = explode('&', $querystring);
    foreach($fields as $field) {
	$pair = array_map('urldecode', explode('=', $field));
	if(isset($pair[0])) {
	    $args[$pair[0]] = default_value($pair[1], '');
	}
    }
    return $args;
}

function url_make($p = '', $h = array()) {
    $url = BASE_URI . '?';
    if($p !== '')
	$url .= url_path_encode($p);
    if(!empty($h))
	$url .= ($p !== '' ? '&' : '') . url_make_query($h);
    return $url;
}

function url_get_site_url() {
    if(BASE_URI !== '')
	return BASE_URI;
    $scheme = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
    $port = $_SERVER['SERVER_PORT'];
    if(($port == 80  && $scheme === 'http') ||
       ($port == 443  && $scheme === 'https'))
	$port = '';
    $path = $_SERVER['REQUEST_URI'];
    $path = preg_replace('/\?.*$/', '', $path);
    return $scheme . '://' . $_SERVER['HTTP_HOST'] . $port . $path;
}

function relative_path($from, $to) {
    $from = str_replace('\\', '/', $from);
    $to = str_replace('\\', '/', $to);
    $froms = explode('/', $from);
    $tos = explode('/', $to);
    $fn = count($froms);
    $tn = count($tos);
    $path = array();
    
    for($i = 0; $i < $fn && $i < $tn - 1 && $froms[$i] == $tos[$i]; $i++);
    $same_index = $i;
    for($i = 0; $i < $fn - $same_index - 1; $i++)
	$path[] = '..';
    for($i = $same_index; $i < $tn; $i++)
	$path[] = $tos[$i];

    if(empty($path))
	$path = './';
    else {
	$last = array_pop($path);
	if(preg_match('_^index\.html?$_', $last))
	    $last = '';
	if($last == '' && empty($path))
	    $path = './';
	else {
	    array_push($path, $last);
	    $path = array_map('urlencode', $path);
	    $path = implode('/', $path);
	}
    }
    return $path;
}

function ouths($str) {
    echo(htmlspecialchars($str));
}

function outraw($str) {
    echo($str);
}

function string_to_list($string, $separator = ' ') {
    $list = explode($separator, language_sanitize($string));
    $list = array_map('trim', $list);
    return array_filter($list, 'strlen');
}

function list_to_string($list, $separator = ' ') {
    return implode($separator, $list);
}

function time_format($t, $type = 'time') {
    if($t == 'now')
	return l('Now');
    if($type == 'date')
	return strftime(DATE_FORMAT, $t);
    return strftime(TIME_FORMAT, $t);
}

function time_xmlformat($time) {
    return date('Y-m-d', $time) . 'T' . date('H:i:sP', $time);
}

function time_get_timezones() {
    $timezones = timezone_abbreviations_list();
    $result = array();
    foreach($timezones as &$timezonegroup) {
	foreach($timezonegroup as &$timezone) {
	    if(!empty($timezone['timezone_id']))
		$result[$timezone['timezone_id']] = $timezone['timezone_id'];
	}
    }
    return $result;
}

define('DATEPAGE_REGEX', '\d{4}\-\d{2}\-\d{2}');

function datepage_format($t) {
    return strftime('%Y-%m-%d', $t);
}

function datepage_is_date($string) {
    return preg_match('|^' . DATEPAGE_REGEX . '$|', $string);
}

function datepage_is_datepage($pagename, &$date = null) {
    if(preg_match('|/(' . DATEPAGE_REGEX . ')$|', $pagename, $maches)) {
	$date = $maches[1];
	return true;
    }
    return false;
}

function options_parse($text) {
    $fields = explode('/', $text);
    $options = array();
    foreach($fields as $field) {
	$kv = explode(':', $field);
	if(count($kv) == 1)
	    $options[$kv[0]] = $kv[0];
	else if(count($kv) > 1) {
	    $options[$kv[1]] = $kv[0];
	}
    }
    return $options;
}

function image_parse_size($arg, &$width, &$height) {
    $is_max = false;

    if(preg_match('/^max *(.*)$/', $arg, $maches)) {
	$is_max = true;
	$arg = $maches[1];
    }

    if(preg_match('/^(\d+)%$/', $arg, $matches)) {
	$width = round($matches[1] * $width / 100.0);
	$height = round($matches[1] * $height / 100.0);
    } else if(preg_match('/^(\d+)x(\d+)!$/', $arg, $matches)) {
	$width = $matches[1];
	$height = $matches[2];
    } else if(preg_match('/^(\d+)x(\d+)$/', $arg, $matches)) {
	if($height * $width > 0) {
	    if($is_max && $width <= $matches[1] && $height <= $matches[2])
		return;
	    $ratio = $height / $width;
	    $width = $matches[1];
	    $height = round($width * $ratio);
	    if($height > $matches[2]) {
		$height = $matches[2];
		$width = round($height / $ratio);
	    }
	} else {
	    $width = $matches[1];
	    $height = $matches[2];
	}
    } else if(preg_match('/^(\d+)x?$/', $arg, $matches)) {
	if($height * $width > 0) {
	    if($is_max && $width <= $matches[1])
		return;
	    $ratio = $height / $width;
	    $width = $matches[1];
	    $height = round($width * $ratio);
	}	    
    } else if(preg_match('/^x(\d+)$/', $arg, $matches)) {
	if($height * $width > 0) {
	    if($is_max && $height <= $matches[1])
		return;
	    $ratio = $height / $width;
	    $height = $matches[1];
	    $width = round($height / $ratio);
	}	    
    }
}

$ob_buffer_writer_handler = null;
function ob_buffer_writer($buffer) {
    global $ob_buffer_writer_handler;
    fwrite($ob_buffer_writer_handler, $buffer);
    return '';
}

function ob_error_handler($str) {
    $error = error_get_last();
    if ($error && $error["type"] == E_USER_ERROR || $error["type"] == E_ERROR) {
	return "$error[message] in $error[file] on line $error[line]\n";
    }
    return $str;
}

function string_insert(&$src, $pos, $text) {
    return substr($src, 0, $pos) . $text . substr($src, $pos);
}

function alpha_to_int($alpha) {
    $alpha = strtoupper($alpha);
    $x = 0;
    $len = strlen($alpha);
    for($i = 0; $i < $len; $i++) {
	$x += (ord($alpha[$i]) - ord('A') + 1) * pow(26, $len - $i - 1);
    }
    $x--;
    return (int)$x;
}

function int_to_alpha($num) {
    $i = 0;
    $alpha = '';
    $fix = 0;
    $num++;
    do {
	$num--;
	$x = ($num % 26);
	$alpha = chr($x + ord('A')) . $alpha;
	$num = floor($num / 26);
	$fix = 1;
    } while($num > 0);
    return $alpha;
}

function common_strwidth($text) {
    if(function_exists('mb_strwidth'))
	return mb_strwidth($text);
    else
	return strlen($text);
}

function rmdir_recursive($dir) {
    $files = new RecursiveIteratorIterator(
	new RecursiveDirectoryIterator($dir),
	RecursiveIteratorIterator::CHILD_FIRST);
    foreach($files as $file) {
	if($file->isDir()) {
	    rmdir($file->getPathname());
	} else {
	    unlink($file->getPathname());
	}
    }
    return rmdir($dir);
}

function run_queue($args, $dom, $show_options, &$queue,
		   $processor, $initializer = null, $finalizer = null, &$func_arg = null) {
    $time_limit = 10;
    $max_execution_time = ini_get('max_execution_time');
    if($max_execution_time * 0.8 < $time_limit)
	$time_limit = $max_execution_time * 0.8;

    $queue_num = count($queue);

    if(empty($args['start_time']))
	$args['start_time'] = time();

    $offset = 0;
    if(!isset($args['offset'])) {
	if(!empty($initializer)) {
	    if($initializer($args, $dom, $func_arg) === false)
		return;
	}
    } else {
	$offset = max(0, (int)$args['offset']);
	while($offset < $queue_num) {
	    if($processor($args, $dom, $func_arg, $queue[$offset]) === false)
		return;
	    $execution_time = execution_time();
	    $offset++;
	    if($time_limit < $execution_time) {
		break;
	    } 
	}
    }

    $p = dom_append_child($dom, 'p');
    $text = lf($show_options['progress'], 
	       array('num' => $offset,
		     'totalnum' => $queue_num,
		     'percent' => $queue_num == 0 ? 100 : round($offset * 100 / $queue_num, 1)));
    dom_append_text($p, $text);
    $progress = dom_append_child($dom, 'progress', '', array('max' => $queue_num, 'value' => $offset));
    dom_append_text($progress, l($show_options['no_progress']));

    $elapsed = time() - $args['start_time'];
    dom_append_text(
	$p, ' ' . lf('Elapsed time $time$',
		     array('time' => strftime('%H:%M:%S', $elapsed - date('Z')))));

    if(!empty($args['done'])) {
	if(!empty($finalizer))
	    $finalizer($args, $dom, $func_arg);
	else {
	    $p = dom_append_child($dom, 'p');
	    dom_append_text($p, l('Done.'));
	}
    } else {
	$new_args = $args;
	$new_args['offset'] = $offset;
	if($offset >= $queue_num)
	    $new_args['done'] = 'true';
	head_tags_add_meta(
	    array('http-equiv' => 'refresh',
		  'content' => '0; url=' . url_make('', $new_args))	    
	    );
    }
}

function print_stacktrace($message = '') {
    $frames = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
    if(!empty($frames)) {
	print("<pre>\n");
	print($message . "\n");
	foreach($frames as $frame) {
	    printf("%20s:%d %s()\n", basename(@$frame['file']), @$frame['line'], $frame['function']);
	}
	print("</pre>\n");
    }
}

if(DEBUG_MODE === 'true') {
    function stacktrace_error_handler($errno,$message,$file,$line,$context) {
	print_stacktrace($message);
	return false;
    }
    set_error_handler("stacktrace_error_handler");

    function debug_log($message) {
	$frames = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	$frame = reset($frames);

	$output = strftime(TIME_FORMAT) . ' ';
	$output .= basename($frame['file']) .':'. $frame['line'] . ' ';
	if(is_scalar($message)) {
	    if(strpos($message, "\n") === false) {
		$output .= $message;
	    } else {
		$output .= "\n" . $message;
	    }
	} else {
	    $output .= "\n" . print_r($message, true);
	}
	$output .= "\n";
	error_log($output, 3, 'debug.log');
    }

    function debug_log_backtrace($message) {
	$frames = debug_backtrace();
	//array_shift($frames);
	$output = $message . "\n";
	if(!empty($frames)) {
	    foreach($frames as $frame) {
		$output .= sprintf("%20s:%d %s()\n", basename(@$frame['file']), @$frame['line'], $frame['function']);
	    }
	}
	debug_log($output);
    }

    $debug_laps = array();
    function debug_watch($name, $facility = 'default') {
	global $debug_laps;
	$debug_laps[$facility][] = array(
	    'name' => $name,
	    'time' => microtime(true),
	    'memory' => memory_get_usage(),
	    );
    }

    function debug_watch_show($facility = 'default') {
	global $debug_laps;
	$output = '<table style="clear: both;">';
	$prelap = array_shift($debug_laps[$facility]);
	foreach($debug_laps[$facility] as $lap) {
	    $output .= sprintf("<tr style=\"text-align:right\">" .
			       "<td>%s</td><td>%.5f</td><td>%s bytes</td><td>%s bytes</td></tr>\n",
			       $lap['name'], 
			       $lap['time'] - $prelap['time'],
			       number_format($lap['memory']),
			       number_format($lap['memory'] - $prelap['memory'])
		);
	    $prelap = $lap;
	}
	$output .= '</table>';
	echo($output);
    }

} else {
    function debug_log($message) {}
    function debug_log_backtrace($frames) {}
    function debug_watch($name, $facility = 'default') {}
    function debug_watch_show($facility = 'default') {}
}
?>
